@@ -5,13 +5,14 @@ import io
 import json
 import re
 
-from cumulusci.tasks.salesforce import BaseSalesforceApiTask
 from cumulusci.core.exceptions import (
-    TaskOptionsError,
     ApexTestException,
     CumulusCIException,
+    TaskOptionsError,
 )
-from cumulusci.core.utils import process_bool_arg, process_list_arg, decode_to_unicode
+from cumulusci.core.utils import decode_to_unicode, process_bool_arg, process_list_arg
+from cumulusci.tasks.salesforce import BaseSalesforceApiTask
+from cumulusci.utils.http.requests_utils import safe_json_from_response
 
 APEX_LIMITS = {
     "Soql": {
@@ -78,7 +79,7 @@ WHERE AsyncApexJobId='{}'
 
 
 class RunApexTests(BaseSalesforceApiTask):
-    """ Task to run Apex tests with the Tooling API and report results.
+    """Task to run Apex tests with the Tooling API and report results.
 
     This task optionally supports retrying unit tests that fail due to
     transitory issues or concurrency-related row locks. To enable retries,
@@ -90,25 +91,24 @@ class RunApexTests(BaseSalesforceApiTask):
     may automatically be retried. Note that retries are supported whether or not
     the org has parallel Apex testing enabled.
 
-    The `retry_always` option modifies this behavior: if a test run fails and
+    The ``retry_always`` option modifies this behavior: if a test run fails and
     any (not all) of the failures match the specified regular expressions,
     all of the failed tests will be retried in serial. This is helpful when
     underlying row locking errors are masked by custom exceptions.
 
-    A useful base configuration for projects wishing to use retries is
+    A useful base configuration for projects wishing to use retries is:
 
-    ```yaml
-            retry_failures:
-                - "unable to obtain exclusive access to this record"
-                - "UNABLE_TO_LOCK_ROW"
-                - "connection was cancelled here"
-            retry_always: True
-    ```
+    .. code-block:: yaml
+
+        retry_failures:
+            - "unable to obtain exclusive access to this record"
+            - "UNABLE_TO_LOCK_ROW"
+            - "connection was cancelled here"
+        retry_always: True
 
     Some projects' unit tests produce so many concurrency errors that
     it's faster to execute the entire run in serial mode than to use retries.
-    Serial and parallel mode are configured in the scratch org definition file.
-"""
+    Serial and parallel mode are configured in the scratch org definition file."""
 
     api_version = "38.0"
     name = "RunApexTests"
@@ -120,7 +120,6 @@ class RunApexTests(BaseSalesforceApiTask):
                 "project__test__name_match from project config. "
                 "Comma-separated list for multiple patterns."
             ),
-            "required": True,
         },
         "test_name_exclude": {
             "description": (
@@ -160,6 +159,20 @@ class RunApexTests(BaseSalesforceApiTask):
             "description": "By default, all failures must match retry_failures to perform "
             "a retry. Set retry_always to True to retry all failed tests if any failure matches."
         },
+        "required_org_code_coverage_percent": {
+            "description": "Require at least X percent code coverage across the org following the test run.",
+            "usage": "--required_org_code_coverage_percent PERCENTAGE",
+        },
+        "required_per_class_code_coverage_percent": {
+            "description": "Require at least X percent code coverage for every class in the org.",
+        },
+        "verbose": {
+            "description": "By default, only failures get detailed output. "
+            "Set verbose to True to see all passed test methods."
+        },
+        "test_suite_names": {
+            "description": "Accepts a comma-separated list of test suite names. Only runs test classes that are part of the test suites specified."
+        },
     }
 
     def _init_options(self, kwargs):
@@ -168,11 +181,16 @@ class RunApexTests(BaseSalesforceApiTask):
         self.options["test_name_match"] = self.options.get(
             "test_name_match", self.project_config.project__test__name_match
         )
-
         self.options["test_name_exclude"] = self.options.get(
             "test_name_exclude", self.project_config.project__test__name_exclude
         )
 
+        self.options["test_suite_names"] = self.options.get(
+            "test_suite_names", self.project_config.project__test__suite__names
+        )
+        if self.options["test_name_match"] is None:
+            self.options["test_name_match"] = ""
+
         if self.options["test_name_exclude"] is None:
             self.options["test_name_exclude"] = ""
 
@@ -188,8 +206,6 @@ class RunApexTests(BaseSalesforceApiTask):
             "json_output", "test_results.json"
         )
 
-        self.options["managed"] = process_bool_arg(self.options.get("managed", False))
-
         self.options["retry_failures"] = process_list_arg(
             self.options.get("retry_failures", [])
         )
@@ -205,11 +221,37 @@ class RunApexTests(BaseSalesforceApiTask):
                 )
         self.options["retry_failures"] = compiled_res
         self.options["retry_always"] = process_bool_arg(
-            self.options.get("retry_always", False)
+            self.options.get("retry_always") or False
         )
 
+        self.verbose = process_bool_arg(self.options.get("verbose") or False)
+
         self.counts = {}
 
+        if "required_org_code_coverage_percent" in self.options:
+            try:
+                self.code_coverage_level = int(
+                    str(self.options["required_org_code_coverage_percent"]).rstrip("%")
+                )
+            except ValueError:
+                raise TaskOptionsError(
+                    f"Invalid code coverage level {self.options['required_org_code_coverage_percent']}"
+                )
+        else:
+            self.code_coverage_level = 0
+
+        self.required_per_class_code_coverage_percent = int(
+            self.options.get("required_per_class_code_coverage_percent", 0)
+        )
+        # Raises a TaskOptionsError when the user provides both test_suite_names and test_name_match.
+        if (self.options["test_suite_names"]) and (
+            self.options["test_name_match"] is not None
+            and self.options["test_name_match"] != "%_TEST%"
+        ):
+            raise TaskOptionsError(
+                "Both test_suite_names and test_name_match cannot be passed simultaneously"
+            )
+
     # pylint: disable=W0201
     def _init_class(self):
         self.classes_by_id = {}
@@ -220,13 +262,19 @@ class RunApexTests(BaseSalesforceApiTask):
         self.retry_details = None
 
     def _get_namespace_filter(self):
-        if self.options["managed"]:
+
+        if self.options.get("managed"):
+
             namespace = self.options.get("namespace")
+
             if not namespace:
                 raise TaskOptionsError(
                     "Running tests in managed mode but no namespace available."
                 )
             namespace = "'{}'".format(namespace)
+        elif self.org_config.namespace:
+            namespace = self.org_config.namespace
+            namespace = "'{}'".format(namespace)
         else:
             namespace = "null"
         return namespace
@@ -249,6 +297,7 @@ class RunApexTests(BaseSalesforceApiTask):
         query = "SELECT Id, Name FROM ApexClass " + "WHERE NamespacePrefix = {}".format(
             namespace
         )
+
         if included_tests:
             query += " AND ({})".format(" OR ".join(included_tests))
         if excluded_tests:
@@ -256,13 +305,78 @@ class RunApexTests(BaseSalesforceApiTask):
         return query
 
     def _get_test_classes(self):
+        # If test_suite_names is provided, execute only tests that are a part of the list of test suites provided.
+        if self.options["test_suite_names"]:
+            test_classes_from_test_suite_names = (
+                self._get_test_classes_from_test_suite_names()
+            )
+            return test_classes_from_test_suite_names
+
+        # test_suite_names is not provided. Fetch all the test classes from the org.
+        else:
+            return self._get_all_test_classes()
+
+    def _get_all_test_classes(self):
+        # Fetches all the test classes from the org.
         query = self._get_test_class_query()
-        # Run the query
-        self.logger.info("Running query: {}".format(query))
+        self.logger.info("Fetching all the test classes...")
         result = self.tooling.query_all(query)
         self.logger.info("Found {} test classes".format(result["totalSize"]))
         return result
 
+    def _get_comma_separated_string_of_items(self, itemlist):
+        # Accepts a list of strings. A formatted string is returned.
+        # Example: Input: ['TestSuite1', 'TestSuite2']      Output: ''TestSuite1','TestSuite2''
+        return ",".join([f"'{item}'" for item in itemlist])
+
+    def _get_test_suite_ids_from_test_suite_names_query(self, test_suite_names_arg):
+        # Returns a query string which when executed fetches the test suite ids of the list of test suite names.
+        test_suite_names = self._get_comma_separated_string_of_items(
+            test_suite_names_arg.split(",")
+        )
+        query1 = f"SELECT Id, TestSuiteName FROM ApexTestSuite WHERE TestSuiteName IN ({test_suite_names})"
+        return query1
+
+    def _get_test_classes_from_test_suite_ids_query(self, testSuiteIds):
+        # Returns a query string which when executed fetches Apex test classes for the given list of test suite ids.
+        # Apex test classes passed under test_name_exclude are ignored.
+        testSuiteIds_formatted = self._get_comma_separated_string_of_items(testSuiteIds)
+
+        if len(testSuiteIds_formatted) == 0:
+            testSuiteIds_formatted = "''"
+
+        test_name_exclude_arg = self.options["test_name_exclude"]
+        condition = ""
+
+        # Check if test_name_exclude is provided. Append to query string if the former is specified.
+        if test_name_exclude_arg:
+            test_name_exclude = self._get_comma_separated_string_of_items(
+                test_name_exclude_arg.split(",")
+            )
+            condition = f"AND Name NOT IN ({test_name_exclude})"
+
+        query = f"SELECT Id, Name FROM ApexClass WHERE Id IN (SELECT ApexClassId FROM TestSuiteMembership WHERE ApexTestSuiteId IN ({testSuiteIds_formatted})) {condition}"
+        return query
+
+    def _get_test_classes_from_test_suite_names(self):
+        # Returns a list of Apex test classes that belong to the test suite(s) specified. Test classes specified in test_name_exclude are excluded.
+        test_suite_names_arg = self.options["test_suite_names"]
+        query1 = self._get_test_suite_ids_from_test_suite_names_query(
+            test_suite_names_arg
+        )
+        self.logger.info("Fetching test suite metadata...")
+        result = self.tooling.query_all(query1)
+        testSuiteIds = []
+
+        for record in result["records"]:
+            testSuiteIds.append(str(record["Id"]))
+
+        query2 = self._get_test_classes_from_test_suite_ids_query(testSuiteIds)
+        self.logger.info("Fetching test classes belonging to the test suite(s)...")
+        result = self.tooling.query_all(query2)
+        self.logger.info("Found {} test classes".format(result["totalSize"]))
+        return result
+
     def _get_test_methods_for_class(self, class_name):
         result = self.tooling.query(
             f"SELECT SymbolTable FROM ApexClass WHERE Name='{class_name}'"
@@ -330,6 +444,13 @@ class RunApexTests(BaseSalesforceApiTask):
                 f"Class {class_name} failed to run some tests with the message {error}. Applying error to unit test results."
             )
 
+            # In Spring '20, we cannot get symbol tables for managed classes.
+            if self.options.get("managed"):
+                self.logger.error(
+                    f"Cannot access symbol table for managed class {class_name}. Failure will not be retried."
+                )
+                continue
+
             # Get all the method names for this class
             test_methods = self._get_test_methods_for_class(class_name)
             for test_method in test_methods:
@@ -373,18 +494,37 @@ class RunApexTests(BaseSalesforceApiTask):
         class_names = list(self.results_by_class_name.keys())
         class_names.sort()
         for class_name in class_names:
-            message = "Class: {}".format(class_name)
-            self.logger.info(message)
+            self.retry_details = {}
+            method_names = list(self.results_by_class_name[class_name].keys())
+            # Added to process for the None methodnames
+
+            if None in method_names:
+                class_id = self.classes_by_name[class_name]
+                self.retry_details.setdefault(class_id, []).append(
+                    self._get_test_methods_for_class(class_name)
+                )
+                del self.results_by_class_name[class_name][None]
+                self.logger.info(
+                    f"Retrying class with id: {class_id} name:{class_name} due to `None` methodname"
+                )
+                self.counts["Retriable"] += len(self.retry_details[class_id])
+                self._attempt_retries()
+
+            has_failures = any(
+                result["Outcome"] in ["Fail", "CompileFail"]
+                for result in self.results_by_class_name[class_name].values()
+            )
+            if has_failures or self.verbose:
+                self.logger.info(f"Class: {class_name}")
             method_names = list(self.results_by_class_name[class_name].keys())
             method_names.sort()
             for method_name in method_names:
                 result = self.results_by_class_name[class_name][method_name]
-                message = "\t{}: {}".format(result["Outcome"], result["MethodName"])
+                message = f"\t{result['Outcome']}: {result['MethodName']}"
                 duration = result["RunTime"]
                 result["stats"] = self._get_stats_from_result(result)
                 if duration:
-                    message += " ({}s)".format(duration)
-                self.logger.info(message)
+                    message += f" ({duration}ms)"
                 test_results.append(
                     {
                         "Children": result.get("children", None),
@@ -398,8 +538,11 @@ class RunApexTests(BaseSalesforceApiTask):
                     }
                 )
                 if result["Outcome"] in ["Fail", "CompileFail"]:
-                    self.logger.info("\tMessage: {}".format(result["Message"]))
-                    self.logger.info("\tStackTrace: {}".format(result["StackTrace"]))
+                    self.logger.info(message)
+                    self.logger.info(f"\tMessage: {result['Message']}")
+                    self.logger.info(f"\tStackTrace: {result['StackTrace']}")
+                elif self.verbose:
+                    self.logger.info(message)
         self.logger.info("-" * 80)
         self.logger.info(
             "Pass: {}  Retried: {}  Fail: {}  CompileFail: {}  Skip: {}".format(
@@ -427,8 +570,8 @@ class RunApexTests(BaseSalesforceApiTask):
                             result["Outcome"],
                         )
                     )
-                    self.logger.error("\tMessage: {}".format(result["Message"]))
-                    self.logger.error("\tStackTrace: {}".format(result["StackTrace"]))
+                    self.logger.error(f"\tMessage: {result['Message']}")
+                    self.logger.error(f"\tStackTrace: {result['StackTrace']}")
 
         return test_results
 
@@ -456,9 +599,23 @@ class RunApexTests(BaseSalesforceApiTask):
         else:
             body = {"classids": ",".join(class_ids)}
 
-        return self.tooling._call_salesforce(
-            method="POST", url=self.tooling.base_url + "runTestsAsynchronous", json=body
-        ).json()
+        return safe_json_from_response(
+            self.tooling._call_salesforce(
+                method="POST",
+                url=self.tooling.base_url + "runTestsAsynchronous",
+                json=body,
+            )
+        )
+
+    def _init_task(self):
+        super()._init_task()
+        if "managed" in self.options:
+            self.options["managed"] = process_bool_arg(self.options["managed"] or False)
+        else:
+            namespace = self.options.get("namespace")
+            self.options["managed"] = (
+                bool(namespace) and namespace in self.org_config.installed_packages
+            )
 
     def _run_task(self):
         result = self._get_test_classes()
@@ -505,6 +662,76 @@ class RunApexTests(BaseSalesforceApiTask):
                 )
             )
 
+        if self.code_coverage_level or self.required_per_class_code_coverage_percent:
+            if self.options.get("namespace") not in self.org_config.installed_packages:
+                self._check_code_coverage()
+            else:
+                self.logger.info(
+                    "This org contains a managed installation; not checking code coverage."
+                )
+        else:
+            self.logger.info(
+                "No code coverage level specified; not checking code coverage."
+            )
+
+    def _check_code_coverage(self):
+        self.logger.info("Checking code coverage.")
+        class_level_coverage_failures = {}
+
+        # Query for Class level code coverage using the aggregate
+        if self.required_per_class_code_coverage_percent:
+            test_classes = self.tooling.query(
+                "SELECT ApexClassOrTrigger.Name, ApexClassOrTriggerId, NumLinesCovered, NumLinesUncovered FROM ApexCodeCoverageAggregate ORDER BY ApexClassOrTrigger.Name ASC"
+            )["records"]
+
+            coverage_percentage = 0
+            for class_level in test_classes:
+                total = (
+                    class_level["NumLinesCovered"] + class_level["NumLinesUncovered"]
+                )
+                # prevent division by 0 errors
+                if total:
+                    # calculate coverage percentage
+                    coverage_percentage = round(
+                        (class_level["NumLinesCovered"] / total) * 100,
+                        2,
+                    )
+
+                if coverage_percentage < self.required_per_class_code_coverage_percent:
+                    class_level_coverage_failures[
+                        class_level["ApexClassOrTrigger"]["Name"]
+                    ] = coverage_percentage
+
+        # Query for OrgWide coverage
+        result = self.tooling.query("SELECT PercentCovered FROM ApexOrgWideCoverage")
+        coverage = result["records"][0]["PercentCovered"]
+
+        errors = []
+        if self.required_per_class_code_coverage_percent:
+            if class_level_coverage_failures:
+                for class_name in class_level_coverage_failures.keys():
+                    errors.append(
+                        f"{class_name}'s code coverage of {class_level_coverage_failures[class_name]}% is below required level of {self.required_per_class_code_coverage_percent}."
+                    )
+            else:
+                self.logger.info(
+                    f"All classes meet code coverage expectations of {self.required_per_class_code_coverage_percent}% ."
+                )
+
+        if coverage < self.code_coverage_level:
+            errors.append(
+                f"Organization-wide code coverage of {coverage}% is below required level of {self.code_coverage_level}"
+            )
+        else:
+            self.logger.info(
+                f"Organization-wide code coverage of {coverage}% meets expectations."
+            )
+
+        if errors:
+            error_message = "\n".join(errors)
+            self.logger.info(error_message)
+            raise ApexTestException(error_message)
+
     def _attempt_retries(self):
         total_method_retries = sum(
             [len(test_list) for test_list in self.retry_details.values()]
@@ -575,34 +802,38 @@ class RunApexTests(BaseSalesforceApiTask):
 
     def _write_output(self, test_results):
         junit_output = self.options["junit_output"]
-        with io.open(junit_output, mode="w", encoding="utf-8") as f:
-            f.write('<testsuite tests="{}">\n'.format(len(test_results)))
-            for result in test_results:
-                s = '  <testcase classname="{}" name="{}"'.format(
-                    result["ClassName"], result["Method"]
-                )
-                if (
-                    "Stats" in result
-                    and result["Stats"]
-                    and "duration" in result["Stats"]
-                ):
-                    s += ' time="{}"'.format(result["Stats"]["duration"])
-                if result["Outcome"] in ["Fail", "CompileFail"]:
-                    s += ">\n"
-                    s += '    <failure type="failed" '
-                    if result["Message"]:
-                        s += 'message="{}"'.format(html.escape(result["Message"]))
-                    s += ">"
+        if junit_output:
+            with io.open(junit_output, mode="w", encoding="utf-8") as f:
+                f.write('<testsuite tests="{}">\n'.format(len(test_results)))
+                for result in test_results:
+                    s = '  <testcase classname="{}" name="{}"'.format(
+                        result["ClassName"], result["Method"]
+                    )
+                    if (
+                        "Stats" in result
+                        and result["Stats"]
+                        and "duration" in result["Stats"]
+                    ):
+                        s += ' time="{}"'.format(result["Stats"]["duration"])
+                    if result["Outcome"] in ["Fail", "CompileFail"]:
+                        s += ">\n"
+                        s += '    <failure type="failed" '
+                        if result["Message"]:
+                            s += 'message="{}"'.format(html.escape(result["Message"]))
+                        s += ">"
 
-                    if result["StackTrace"]:
-                        s += "<![CDATA[{}]]>".format(html.escape(result["StackTrace"]))
-                    s += "</failure>\n"
-                    s += "  </testcase>\n"
-                else:
-                    s += " />\n"
-                f.write(str(s))
-            f.write("</testsuite>")
+                        if result["StackTrace"]:
+                            s += "<![CDATA[{}]]>".format(
+                                html.escape(result["StackTrace"])
+                            )
+                        s += "</failure>\n"
+                        s += "  </testcase>\n"
+                    else:
+                        s += " />\n"
+                    f.write(str(s))
+                f.write("</testsuite>")
 
         json_output = self.options["json_output"]
-        with io.open(json_output, mode="w", encoding="utf-8") as f:
-            f.write(str(json.dumps(test_results, indent=4)))
+        if json_output:
+            with io.open(json_output, mode="w", encoding="utf-8") as f:
+                f.write(str(json.dumps(test_results, indent=4)))
