@@ -1,19 +1,22 @@
 import http.client
+import urllib.parse
 from unittest import mock
-import unittest
 
+import pytest
 import responses
 
-from cumulusci.tasks.release_notes.exceptions import GithubIssuesError
-from cumulusci.tasks.release_notes.generator import GithubReleaseNotesGenerator
-from cumulusci.tasks.release_notes.parser import BaseChangeNotesParser
-from cumulusci.tasks.release_notes.parser import ChangeNotesLinesParser
-from cumulusci.tasks.release_notes.parser import GithubIssuesParser
-from cumulusci.tasks.release_notes.parser import GithubLinesParser
-from cumulusci.tasks.release_notes.parser import IssuesParser
 from cumulusci.core.exceptions import GithubApiNotFoundError
 from cumulusci.core.github import get_github_api
 from cumulusci.tasks.github.tests.util_github_api import GithubApiTestMixin
+from cumulusci.tasks.release_notes.generator import GithubReleaseNotesGenerator
+from cumulusci.tasks.release_notes.parser import (
+    BaseChangeNotesParser,
+    ChangeNotesLinesParser,
+    GithubIssuesParser,
+    GithubLinesParser,
+    InstallLinkParser,
+    IssuesParser,
+)
 from cumulusci.tasks.release_notes.tests.utils import MockUtil
 
 PARSER_CONFIG = [
@@ -32,63 +35,63 @@ PARSER_CONFIG = [
 ]
 
 
-class TestBaseChangeNotesParser(unittest.TestCase):
+class TestBaseChangeNotesParser:
     def test_parse(self):
         parser = BaseChangeNotesParser("Title")
-        with self.assertRaises(NotImplementedError):
+        with pytest.raises(NotImplementedError):
             parser.parse()
 
     def test_render(self):
         parser = BaseChangeNotesParser("Title")
-        with self.assertRaises(NotImplementedError):
+        with pytest.raises(NotImplementedError):
             parser.render()
 
 
-class TestChangeNotesLinesParser(unittest.TestCase):
-    def setUp(self):
+class TestChangeNotesLinesParser:
+    def setup_method(self):
         self.title = "Title"
 
     def test_parse_no_start_line(self):
         change_note = "foo\r\nbar\r\n"
         parser = ChangeNotesLinesParser(None, self.title)
         line_added = parser.parse(change_note)
-        self.assertEqual(parser.content, [])
-        self.assertFalse(line_added)
+        assert parser.content == []
+        assert not line_added
 
     def test_parse_start_line_no_content(self):
         change_note = "# {}\r\n\r\n".format(self.title)
         parser = ChangeNotesLinesParser(None, self.title)
         line_added = parser.parse(change_note)
-        self.assertEqual(parser.content, [])
-        self.assertFalse(line_added)
+        assert parser.content == []
+        assert not line_added
 
     def test_parse_start_line_no_end_line(self):
         change_note = "# {}\r\nfoo\r\nbar".format(self.title)
         parser = ChangeNotesLinesParser(None, self.title)
         line_added = parser.parse(change_note)
-        self.assertEqual(parser.content, ["foo", "bar"])
-        self.assertEqual(True, line_added)
+        assert parser.content == ["foo", "bar"]
+        assert line_added is True
 
     def test_parse_start_line_end_at_header(self):
         change_note = "# {}\r\nfoo\r\n# Another Header\r\nbar".format(self.title)
         parser = ChangeNotesLinesParser(None, self.title)
         line_added = parser.parse(change_note)
-        self.assertEqual(parser.content, ["foo"])
-        self.assertTrue(line_added)
+        assert parser.content == ["foo"]
+        assert line_added
 
     def test_parse_start_line_no_content_no_end_line(self):
         change_note = "# {}".format(self.title)
         parser = ChangeNotesLinesParser(None, self.title)
         line_added = parser.parse(change_note)
-        self.assertEqual(parser.content, [])
-        self.assertFalse(line_added)
+        assert parser.content == []
+        assert not line_added
 
     def test_parse_multiple_start_lines_without_end_lines(self):
         change_note = "# {0}\r\nfoo\r\n# {0}\r\nbar\r\n".format(self.title)
         parser = ChangeNotesLinesParser(None, self.title)
         line_added = parser.parse(change_note)
-        self.assertEqual(parser.content, ["foo", "bar"])
-        self.assertTrue(line_added)
+        assert parser.content == ["foo", "bar"]
+        assert line_added
 
     def test_parse_multiple_start_lines_with_end_lines(self):
         change_note = "# {0}\r\nfoo\r\n\r\n# {0}\r\nbar\r\n\r\nincluded\r\n\r\n# not included".format(
@@ -96,8 +99,8 @@ class TestChangeNotesLinesParser(unittest.TestCase):
         )
         parser = ChangeNotesLinesParser(None, self.title)
         line_added = parser.parse(change_note)
-        self.assertEqual(parser.content, ["foo", "bar", "included"])
-        self.assertTrue(line_added)
+        assert parser.content == ["foo", "bar", "included"]
+        assert line_added
 
     def test_parse_multi_level_indent(self):
         change_note = "# {0}\r\nfoo \r\n    bar  \r\n        baz \r\n".format(
@@ -105,48 +108,53 @@ class TestChangeNotesLinesParser(unittest.TestCase):
         )
         parser = ChangeNotesLinesParser(None, self.title)
         line_added = parser.parse(change_note)
-        self.assertEqual(parser.content, ["foo", "    bar", "        baz"])
-        self.assertTrue(line_added)
+        assert parser.content == ["foo", "    bar", "        baz"]
+        assert line_added
 
     def test_parse_subheading(self):
         change_note = "# {0}\r\n## Subheading\r\nfoo".format(self.title)
         parser = ChangeNotesLinesParser(None, self.title)
         line_added = parser.parse(change_note)
-        self.assertEqual([], parser.content)
-        self.assertEqual({"Subheading": ["foo"]}, parser.h2)
-        self.assertTrue(line_added)
+        assert [] == parser.content
+        assert {"Subheading": ["foo"]} == parser.h2
+        assert line_added
+
+    def test_parse_subheading_from_another_section(self):
+        change_note = "## Subheading\r\n# {0}\r\nfoo".format(self.title)
+        parser = ChangeNotesLinesParser(None, self.title)
+        line_added = parser.parse(change_note)
+        assert ["foo"] == parser.content
+        assert {} == parser.h2
+        assert line_added
 
     def test_render_no_content(self):
         parser = ChangeNotesLinesParser(None, self.title)
-        self.assertEqual(parser.render(), None)
+        assert parser.render() == ""
 
     def test_render_one_content(self):
         parser = ChangeNotesLinesParser(None, self.title)
         content = ["foo"]
         parser.content = content
-        self.assertEqual(
-            parser.render(), "# {}\r\n\r\n{}".format(self.title, content[0])
-        )
+        assert parser.render() == "# {}\r\n\r\n{}".format(self.title, content[0])
 
     def test_render_multiple_content(self):
         parser = ChangeNotesLinesParser(None, self.title)
         content = ["foo", "bar"]
         parser.content = content
-        self.assertEqual(
-            parser.render(), "# {}\r\n\r\n{}".format(self.title, "\r\n".join(content))
+        assert parser.render() == "# {}\r\n\r\n{}".format(
+            self.title, "\r\n".join(content)
         )
 
     def test_render_subheadings(self):
         parser = ChangeNotesLinesParser(None, self.title)
         parser.h2 = {"Subheading": ["foo"]}
-        self.assertEqual(
-            parser.render(),
-            "# {}\r\n\r\n\r\n## Subheading\r\n\r\nfoo".format(self.title),
+        assert parser.render() == "# {}\r\n\r\n\r\n## Subheading\r\n\r\nfoo".format(
+            self.title
         )
 
 
-class TestGithubLinesParser(unittest.TestCase):
-    def setUp(self):
+class TestGithubLinesParser:
+    def setup_method(self):
         self.title = "Title"
 
     def test_parse(self):
@@ -156,9 +164,9 @@ class TestGithubLinesParser(unittest.TestCase):
             number=1, html_url="http://pr", body="# {}\r\n\r\nfoo".format(self.title)
         )
         parser.parse(pr)
-        self.assertEqual(1, parser.pr_number)
-        self.assertEqual("http://pr", parser.pr_url)
-        self.assertEqual(["foo [[PR1](http://pr)]"], parser.content)
+        assert 1 == parser.pr_number
+        assert parser.pr_url == "http://pr"
+        assert ["foo [[PR1](http://pr)]"] == parser.content
 
     def test_parse_empty_pull_request_body(self):
         generator = mock.Mock(link_pr=True)
@@ -168,15 +176,15 @@ class TestGithubLinesParser(unittest.TestCase):
         assert not line_added
 
 
-class TestIssuesParser(unittest.TestCase):
-    def setUp(self):
+class TestIssuesParser:
+    def setup_method(self):
         self.title = "Issues"
 
     def test_issue_numbers(self):
         change_note = "# {}\r\nfix #2\r\nfix #3\r\nfix #5\r\n".format(self.title)
         parser = IssuesParser(None, self.title)
         parser.parse(change_note)
-        self.assertEqual(parser.content, [2, 3, 5])
+        assert parser.content == [2, 3, 5]
 
     def test_issue_numbers_with_links(self):
         change_note = "# {}\r\nfix [#2](https://issue)\r\nfix [#3](http://issue)\r\nfix #5\r\n".format(
@@ -184,13 +192,13 @@ class TestIssuesParser(unittest.TestCase):
         )
         parser = IssuesParser(None, self.title)
         parser.parse(change_note)
-        self.assertEqual(parser.content, [2, 3, 5])
+        assert parser.content == [2, 3, 5]
 
     def test_issue_numbers_and_other_numbers(self):
         change_note = "# {}\r\nfixes #2 but not # 3 or 5".format(self.title)
         parser = IssuesParser(None, self.title)
         parser.parse(change_note)
-        self.assertEqual(parser.content, [2])
+        assert parser.content == [2]
 
     def test_multiple_issue_numbers_per_line(self):
         change_note = "# {}\r\nfix #2 also does fix #3 and fix #5\r\n".format(
@@ -198,16 +206,16 @@ class TestIssuesParser(unittest.TestCase):
         )
         parser = IssuesParser(None, self.title)
         parser.parse(change_note)
-        self.assertEqual(parser.content, [2, 3, 5])
+        assert parser.content == [2, 3, 5]
 
     def test_render(self):
         parser = IssuesParser(None, self.title)
         parser.content = ["1: foo"]
-        self.assertEqual("# Issues\r\n\r\n#1: foo", parser.render())
+        assert parser.render() == "# Issues\r\n\r\n#1: foo"
 
 
-class TestGithubIssuesParser(unittest.TestCase, GithubApiTestMixin):
-    def setUp(self):
+class TestGithubIssuesParser(GithubApiTestMixin):
+    def setup_method(self):
         self.init_github()
         self.gh = get_github_api("TestUser", "TestPass")
         self.title = "Issues"
@@ -232,7 +240,7 @@ class TestGithubIssuesParser(unittest.TestCase, GithubApiTestMixin):
         parser.parse(pull_request)
         pr_url = "https://github.com/TestOwner/TestRepo/pulls/{}".format(self.pr_number)
         expected_content = self._create_expected_content([2, 3, 5], pr_url)
-        self.assertEqual(parser.content, expected_content)
+        assert parser.content == expected_content
 
     @responses.activate
     def test_issue_numbers_and_other_numbers(self):
@@ -246,7 +254,7 @@ class TestGithubIssuesParser(unittest.TestCase, GithubApiTestMixin):
         parser.parse(pull_request)
         pr_url = "https://github.com/TestOwner/TestRepo/pulls/{}".format(self.pr_number)
         expected_content = self._create_expected_content([2], pr_url)
-        self.assertEqual(parser.content, expected_content)
+        assert parser.content == expected_content
 
     @responses.activate
     def test_no_issue_numbers(self):
@@ -259,7 +267,7 @@ class TestGithubIssuesParser(unittest.TestCase, GithubApiTestMixin):
         pull_request = repo.pull_request(pr_number)
         parser = GithubIssuesParser(generator, self.title)
         parser.parse(pull_request)
-        self.assertEqual(parser.content, [])
+        assert parser.content == []
 
     @responses.activate
     def test_render_issue_number_valid(self):
@@ -280,7 +288,7 @@ class TestGithubIssuesParser(unittest.TestCase, GithubApiTestMixin):
         expected_render = self._create_expected_render(
             self.issue_number_valid, expected_response["title"], True
         )
-        self.assertEqual(parser.render(), expected_render)
+        assert parser.render() == expected_render
 
     @responses.activate
     def test_render_issue_number_invalid(self):
@@ -302,13 +310,20 @@ class TestGithubIssuesParser(unittest.TestCase, GithubApiTestMixin):
                 "pr_url": self.pr_url,
             }
         ]
-        with self.assertRaises(GithubApiNotFoundError):
+        with pytest.raises(GithubApiNotFoundError):
             parser.render()
 
+    def test_init__issues_enabled(self):
+        generator = mock.Mock(has_issues=True)
+        result = GithubIssuesParser(generator, self.title)
+        assert isinstance(result, GithubIssuesParser)
+        assert result.title == self.title
+
     def test_init__issues_disabled(self):
         generator = mock.Mock(has_issues=False)
-        with self.assertRaises(GithubIssuesError):
-            GithubIssuesParser(generator, self.title)
+        result = GithubIssuesParser(generator, self.title)
+        assert isinstance(result, GithubLinesParser)
+        assert result.title == self.title
 
     def _create_expected_content(self, issue_numbers, pr_url):
         y = []
@@ -329,8 +344,8 @@ class TestGithubIssuesParser(unittest.TestCase, GithubApiTestMixin):
         return generator
 
 
-class TestCommentingGithubIssuesParser(unittest.TestCase, GithubApiTestMixin):
-    def setUp(self):
+class TestCommentingGithubIssuesParser(GithubApiTestMixin):
+    def setup_method(self):
         self.init_github()
         self.gh = get_github_api("TestUser", "TestPass")
         self.mock_util = MockUtil("TestOwner", "TestRepo")
@@ -387,8 +402,8 @@ class TestCommentingGithubIssuesParser(unittest.TestCase, GithubApiTestMixin):
             issue_number, expected_issue["title"], False
         )
         render = parser.render()
-        self.assertEqual(render, expected_render)
-        self.assertEqual(len(responses.calls._calls), 2)
+        assert render == expected_render
+        assert len(responses.calls._calls) == 2
 
     @responses.activate
     def test_render_issue_with_beta_comment(self):
@@ -423,8 +438,8 @@ class TestCommentingGithubIssuesParser(unittest.TestCase, GithubApiTestMixin):
             issue_number, expected_issue["title"], False
         )
         render = parser.render()
-        self.assertEqual(render, expected_render)
-        self.assertEqual(len(responses.calls._calls), 3)
+        assert render == expected_render
+        assert len(responses.calls._calls) == 3
 
     @responses.activate
     def test_render_issue_without_beta_comment(self):
@@ -464,8 +479,8 @@ class TestCommentingGithubIssuesParser(unittest.TestCase, GithubApiTestMixin):
         expected_render = self._create_expected_render(
             issue_number, expected_issue["title"], False
         )
-        self.assertEqual(parser.render(), expected_render)
-        self.assertEqual(len(responses.calls._calls), 4)
+        assert parser.render() == expected_render
+        assert len(responses.calls._calls) == 4
 
     @responses.activate
     def test_render_issue_with_prod_comment(self):
@@ -499,8 +514,8 @@ class TestCommentingGithubIssuesParser(unittest.TestCase, GithubApiTestMixin):
         expected_render = self._create_expected_render(
             issue_number, expected_issue["title"], False
         )
-        self.assertEqual(parser.render(), expected_render)
-        self.assertEqual(len(responses.calls._calls), 3)
+        assert parser.render() == expected_render
+        assert len(responses.calls._calls) == 3
 
     @responses.activate
     def test_render_issue_without_prod_comment(self):
@@ -541,11 +556,270 @@ class TestCommentingGithubIssuesParser(unittest.TestCase, GithubApiTestMixin):
             issue_number, expected_issue["title"], False
         )
         render = parser.render()
-        self.assertEqual(render, expected_render)
-        self.assertEqual(len(responses.calls._calls), 4)
+        assert render == expected_render
+        assert len(responses.calls._calls) == 4
 
     def _create_expected_render(self, issue_number, issue_title, link_pr):
         render = "# {}\r\n\r\n#{}: {}".format(self.title, issue_number, issue_title)
         if link_pr:
             render += " [[PR{}]({})]".format(self.pr_number, self.pr_url)
         return render
+
+
+class TestInstallLinkParser:
+    def test_no_package_version(self):
+        generator = mock.Mock(
+            link_pr=True,
+            version_id=None,
+            sandbox_date=None,  # need to set explicitly due to mock, will default to None when using CLI
+            production_date=None,  # need to set explicitly due to mock, will default to None when using CLI
+            trial_info=False,  # need to set explicitly due to mock, will default to False when using CLI)
+        )
+        parser = InstallLinkParser(generator, "Title")
+        parser.parse("abc")
+        assert parser.render() == ""
+
+    def test_package_with_version_id_no_dates_no_trial(self):
+        generator = mock.Mock(
+            link_pr=True,
+            version_id="04t0000asdf",
+            sandbox_date=None,  # need to set explicitly due to mock, will default to None when using CLI
+            production_date=None,  # need to set explicitly due to mock, will default to None when using CLI
+            trial_info=False,  # need to set explicitly due to mock, will default to False when using CLI
+        )
+        parser = InstallLinkParser(generator, "Title")
+        parser.parse("abc")
+        version_id = urllib.parse.quote_plus(generator.version_id)
+        assert (
+            f"""# Title\r\n\r\nSandbox & Scratch Orgs:\r\nhttps://test.salesforce.com/packaging/installPackage.apexp?p0={version_id}\r\n\r\nProduction & Developer Edition Orgs:\r\nhttps://login.salesforce.com/packaging/installPackage.apexp?p0={version_id}"""
+            == parser.render()
+        )
+
+    def test_package_version_id_both_dates_no_trial(self):
+        generator = mock.Mock(
+            link_pr=True,
+            version_id="04t0000asdf",
+            sandbox_date="2020-10-10",
+            production_date="2020-10-11",
+            trial_info=False,
+        )  # need to set explicitly due to mock, will default to False when using CLI
+
+        parser = InstallLinkParser(generator, "Title")
+        parser.parse("abc")
+        version_id = urllib.parse.quote_plus(generator.version_id)
+        assert (
+            f"""# Title\r\n\r\n## Push Schedule\r\nSandbox orgs: {generator.sandbox_date}\r\nProduction orgs: {generator.production_date}\r\n\r\nSandbox & Scratch Orgs:\r\nhttps://test.salesforce.com/packaging/installPackage.apexp?p0={version_id}\r\n\r\nProduction & Developer Edition Orgs:\r\nhttps://login.salesforce.com/packaging/installPackage.apexp?p0={version_id}"""
+            == parser.render()
+        )
+
+    def test_package_version_id_sandbox_date_no_trial(self):
+        generator = mock.Mock(
+            link_pr=True,
+            version_id="04t0000asdf",
+            sandbox_date="2020-10-10",
+            production_date=None,  # need to set explicitly due to mock, will default to False when using CLI
+            trial_info=False,  # need to set explicitly due to mock, will default to False when using CLI
+        )
+        parser = InstallLinkParser(generator, "Title")
+        parser.parse("abc")
+        version_id = urllib.parse.quote_plus(generator.version_id)
+        assert (
+            f"""# Title\r\n\r\n## Push Schedule\r\nSandbox orgs: {generator.sandbox_date}\r\n\r\nSandbox & Scratch Orgs:\r\nhttps://test.salesforce.com/packaging/installPackage.apexp?p0={version_id}\r\n\r\nProduction & Developer Edition Orgs:\r\nhttps://login.salesforce.com/packaging/installPackage.apexp?p0={version_id}"""
+            == parser.render()
+        )
+
+    def test_package_version_id_production_date_no_trial(self):
+        generator = mock.Mock(
+            link_pr=True,
+            version_id="04t0000asdf",
+            sandbox_date=None,  # need to set explicitly due to mock, will default to None when using CLI
+            production_date="2020-10-10",
+            trial_info=False,  # need to set explicitly due to mock, will default to False when using CLI
+        )
+        parser = InstallLinkParser(generator, "Title")
+        parser.parse("abc")
+        version_id = urllib.parse.quote_plus(generator.version_id)
+        assert (
+            f"""# Title\r\n\r\n## Push Schedule\r\nProduction orgs: {generator.production_date}\r\n\r\nSandbox & Scratch Orgs:\r\nhttps://test.salesforce.com/packaging/installPackage.apexp?p0={version_id}\r\n\r\nProduction & Developer Edition Orgs:\r\nhttps://login.salesforce.com/packaging/installPackage.apexp?p0={version_id}"""
+            == parser.render()
+        )
+
+    def test_package_with_version_id_no_dates_trial(self):
+        generator = mock.Mock(
+            link_pr=True,
+            version_id="04t0000asdf",
+            sandbox_date=None,  # need to set explicitly due to mock, will default to None when using CLI
+            production_date=None,  # need to set explicitly due to mock, will default to None when using CLI
+            trial_info="`TBD`",
+        )
+        parser = InstallLinkParser(generator, "Title")
+        parser.parse("abc")
+        version_id = urllib.parse.quote_plus(generator.version_id)
+        assert (
+            f"""# Title\r\n\r\nSandbox & Scratch Orgs:\r\nhttps://test.salesforce.com/packaging/installPackage.apexp?p0={version_id}\r\n\r\nProduction & Developer Edition Orgs:\r\nhttps://login.salesforce.com/packaging/installPackage.apexp?p0={version_id}\r\n\r\n## Trialforce Template ID\r\n`TBD`"""
+            == parser.render()
+        )
+
+    def test_package_version_id_both_dates_with_trial(self):
+        generator = mock.Mock(
+            link_pr=True,
+            version_id="04t0000asdf",
+            sandbox_date="2020-10-10",
+            production_date="2020-10-11",
+            trial_info="`TBD`",
+        )
+        parser = InstallLinkParser(generator, "Title")
+        parser.parse("abc")
+        version_id = urllib.parse.quote_plus(generator.version_id)
+        assert (
+            f"""# Title\r\n\r\n## Push Schedule\r\nSandbox orgs: {generator.sandbox_date}\r\nProduction orgs: {generator.production_date}\r\n\r\nSandbox & Scratch Orgs:\r\nhttps://test.salesforce.com/packaging/installPackage.apexp?p0={version_id}\r\n\r\nProduction & Developer Edition Orgs:\r\nhttps://login.salesforce.com/packaging/installPackage.apexp?p0={version_id}\r\n\r\n## Trialforce Template ID\r\n`TBD`"""
+            == parser.render()
+        )
+
+    def test_package_version_id_sandbox_date_with_trial(self):
+        generator = mock.Mock(
+            link_pr=True,
+            version_id="04t0000asdf",
+            sandbox_date="2020-10-10",
+            production_date=None,  # need to set explicitly due to mock, will default to None when using CLI
+            trial_info="`TBD`",
+        )
+        parser = InstallLinkParser(generator, "Title")
+        parser.parse("abc")
+        version_id = urllib.parse.quote_plus(generator.version_id)
+        assert (
+            f"""# Title\r\n\r\n## Push Schedule\r\nSandbox orgs: {generator.sandbox_date}\r\n\r\nSandbox & Scratch Orgs:\r\nhttps://test.salesforce.com/packaging/installPackage.apexp?p0={version_id}\r\n\r\nProduction & Developer Edition Orgs:\r\nhttps://login.salesforce.com/packaging/installPackage.apexp?p0={version_id}\r\n\r\n## Trialforce Template ID\r\n`TBD`"""
+            == parser.render()
+        )
+
+    def test_package_version_id_production_date_with_trial(self):
+        generator = mock.Mock(
+            link_pr=True,
+            version_id="04t0000asdf",
+            sandbox_date=None,  # need to set explicitly due to mock, will default to None when using CLI
+            production_date="2020-10-10",
+            trial_info=False,  # need to set explicitly due to mock, will default to False when using CLI
+        )
+        parser = InstallLinkParser(generator, "Title")
+        parser.parse("abc")
+        version_id = urllib.parse.quote_plus(generator.version_id)
+        assert f"""# Title\r\n\r\n## Push Schedule\r\nProduction orgs: {generator.production_date}\r\n\r\n\r\nProduction & Developer Edition Orgs:\r\nhttps://login.salesforce.com/packaging/installPackage.apexp?p0={version_id}\r\n\r\nSandbox & Scratch Orgs:\r\nhttps://test.salesforce.com/packaging/installPackage.apexp?p0={version_id}\r\n\r\n## Trialforce Template ID\r\n`TBD`"""
+
+    def test_package_no_version_id_sandbox_date_no_trial(self):
+        generator = mock.Mock(
+            link_pr=True,
+            version_id="",
+            sandbox_date="2020-10-10",  # need to set explicitly due to mock, will default to None when using CLI
+            production_date=None,  # need to set explicitly due to mock, will default to None when using CLI
+            trial_info=False,  # need to set explicitly due to mock, will default to False when using CLI
+        )
+        parser = InstallLinkParser(generator, "Title")
+        parser.parse("abc")
+        assert (
+            f"""# Title\r\n\r\n## Push Schedule\r\nSandbox orgs: {generator.sandbox_date}"""
+            == parser.render()
+        )
+
+    def test_package_no_version_id_production_date_no_trial(self):
+        generator = mock.Mock(
+            link_pr=True,
+            version_id="",
+            sandbox_date=None,  # need to set explicitly due to mock, will default to None when using CLI
+            production_date="2020-10-10",
+            trial_info=False,  # need to set explicitly due to mock, will default to False when using CLI
+        )
+        parser = InstallLinkParser(generator, "Title")
+        parser.parse("abc")
+        assert (
+            f"""# Title\r\n\r\n## Push Schedule\r\nProduction orgs: {generator.production_date}"""
+            == parser.render()
+        )
+
+    def test_package_no_version_id_both_dates_no_trial(self):
+        generator = mock.Mock(
+            link_pr=True,
+            version_id="",
+            sandbox_date="2020-10-10",
+            production_date="2020-10-11",
+            trial_info=False,
+        )
+        parser = InstallLinkParser(generator, "Title")
+        parser.parse("abc")
+        assert (
+            f"""# Title\r\n\r\n## Push Schedule\r\nSandbox orgs: {generator.sandbox_date}\r\nProduction orgs: {generator.production_date}"""
+            == parser.render()
+        )
+
+    def test_package_no_version_id_sandbox_date_trial(self):
+        generator = mock.Mock(
+            link_pr=True,
+            version_id="",
+            sandbox_date="2020-10-10",  # need to set explicitly due to mock, will default to None when using CLI
+            production_date=None,  # need to set explicitly due to mock, will default to None when using CLI
+            trial_info="`TBD`",  # need to set explicitly due to mock, will default to False when using CLI
+        )
+        parser = InstallLinkParser(generator, "Title")
+        parser.parse("abc")
+        assert (
+            f"""# Title\r\n\r\n## Push Schedule\r\nSandbox orgs: {generator.sandbox_date}\r\n\r\n## Trialforce Template ID\r\n`TBD`"""
+            == parser.render()
+        )
+
+    def test_package_no_version_id_production_date_trial(self):
+        generator = mock.Mock(
+            link_pr=True,
+            version_id="",
+            sandbox_date=None,  # need to set explicitly due to mock, will default to None when using CLI
+            production_date="2020-10-10",
+            trial_info="`TBD`",  # need to set explicitly due to mock, will default to False when using CLI
+        )
+        parser = InstallLinkParser(generator, "Title")
+        parser.parse("abc")
+        assert (
+            f"""# Title\r\n\r\n## Push Schedule\r\nProduction orgs: {generator.production_date}\r\n\r\n## Trialforce Template ID\r\n`TBD`"""
+            == parser.render()
+        )
+
+    def test_package_no_version_id_both_dates_trial(self):
+        generator = mock.Mock(
+            link_pr=True,
+            version_id="",
+            sandbox_date="2020-10-10",
+            production_date="2020-10-11",
+            trial_info="`TBD`",
+        )
+        parser = InstallLinkParser(generator, "Title")
+        parser.parse("abc")
+        assert (
+            f"""# Title\r\n\r\n## Push Schedule\r\nSandbox orgs: {generator.sandbox_date}\r\nProduction orgs: {generator.production_date}\r\n\r\n## Trialforce Template ID\r\n`TBD`"""
+            == parser.render()
+        )
+
+    def test_package_no_version_no_dates_trial(self):
+        generator = mock.Mock(
+            link_pr=True,
+            version_id="",
+            sandbox_date=None,  # need to set explicitly due to mock, will default to None when using CLI
+            production_date=None,  # need to set explicitly due to mock, will default to None when using CLI
+            trial_info="`TBD`",
+        )
+        parser = InstallLinkParser(generator, "Title")
+        parser.parse("abc")
+        assert (
+            """# Title\r\n\r\n## Trialforce Template ID\r\n`TBD`""" == parser.render()
+        )
+
+    def test_package_version(self):
+        generator = mock.Mock(link_pr=True)
+        generator.version_id = "foo bar"
+        parser = InstallLinkParser(generator, "Title")
+        parser.parse("abc")
+        output = parser.render()
+        assert (
+            "https://login.salesforce.com/packaging/installPackage.apexp?p0=foo+bar"
+            in output
+        )
+        assert (
+            "https://test.salesforce.com/packaging/installPackage.apexp?p0=foo+bar"
+            in output
+        )
